/*
** File: corevpd.c  
** Project: ADB3 core driver
** Purpose: Implements functions for VPD read and write in Core Interface.
**
** (C) Copyright Alpha Data 2009-2010
*/

#include <df.h>

#include "corenotify.h"

static Adb3CoreNotificationList*
getList(
  Adb3CoreDeviceContext* pDevCtx,
  unsigned int notifyType)
{
  Adb3CoreNotificationList* pList;
  unsigned int index;

  if (notifyType >= CORE_NOTIFY_FPGA_INTERRUPT(0) && notifyType < CORE_NOTIFY_FPGA_INTERRUPT(pDevCtx->info.bootstrap.numTargetFpga)) {
    index = notifyType - CORE_NOTIFY_FPGA_INTERRUPT(0);
    pList = &pDevCtx->notification.fpgaInterrupt[index];
  } else if (notifyType >= CORE_NOTIFY_FPGA_ALERT(0) && notifyType < CORE_NOTIFY_FPGA_ALERT(pDevCtx->info.bootstrap.numTargetFpga)) {
    index = notifyType - CORE_NOTIFY_FPGA_ALERT(0);
    pList = &pDevCtx->notification.fpgaAlert[index];
  } else if (notifyType == CORE_NOTIFY_SENSOR_POLL) {
    pList = &pDevCtx->notification.sensorPoll;
  } else {
    /* Invalid notification type */
    return NULL;
  }
  return pList;
}

void
coreNotifyListeners(
  Adb3CoreDeviceContext* pDevCtx,
  Adb3CoreNotificationList* pList)
{
  CoreNotification* pNotification;
  DfListNode* pNode;
  DfSpinLockFlags f;

  /* TO DO - don't invoke callback while holding a lock */
  f = dfSpinLockGet(&pList->lock);
  pNode = dfListGetHead(&pList->header);
  while (NULL != pNode) {
    pNotification = DF_CONTAINER_OF(pNode, CoreNotification, system.node);
    pNotification->system.pCallback(pNotification, pNotification->system.notifyType, pNotification->system.pCallbackContext);
    pNode = dfListGetNext(&pList->header, pNode);
  }
  dfSpinLockPut(&pList->lock, f);
}

boolean_t
coreNotificationReg(
  void* pInterfaceContext,
  unsigned int notifyType,
  CoreNotification* pNotification,
  CoreNotificationCallback* pCallback,
  void* pCallbackContext)
{
  Adb3CoreDeviceContext* pDevCtx = (Adb3CoreDeviceContext*)pInterfaceContext;
  Adb3CoreNotificationList* pList;
  DfSpinLockFlags f;

  pNotification->system.notifyType = notifyType;
  pNotification->system.pCallback = pCallback;
  pNotification->system.pCallbackContext = pCallbackContext;

  pList = getList(pDevCtx, notifyType);
  if (NULL == pList) {
    /* Invalid notification type */
    return FALSE;
  }

  /* Add this node to the linked list */
  f = dfSpinLockGet(&pList->lock);
  dfListAddToTail(&pList->header, &pNotification->system.node);
  dfSpinLockPut(&pList->lock, f);

  return TRUE;
}

boolean_t
coreNotificationUnreg(
  void* pInterfaceContext,
  CoreNotification* pNotification)
{
  Adb3CoreDeviceContext* pDevCtx = (Adb3CoreDeviceContext*)pInterfaceContext;
  Adb3CoreNotificationList* pList;
  unsigned int notifyType;
  DfSpinLockFlags f;

  notifyType = pNotification->system.notifyType;
  pList = getList(pDevCtx, notifyType);
  if (NULL == pList) {
    /* Invalid notification type */
    return FALSE;
  }

  /* Remove this node from the linked list */
  f = dfSpinLockGet(&pList->lock);
  dfListRemove(&pNotification->system.node);
  dfSpinLockPut(&pList->lock, f);

  return TRUE;
}
